@@ -1,6 +1,7 @@
 package Spreadsheet::ParseODS;
 use strict;
 use warnings;
+use 5.010; # for "state"
 
 use Archive::Zip ':ERROR_CODES';
 use Moo 2;
@@ -10,7 +11,7 @@ use Carp qw(croak);
 use List::Util 'max';
 #use Storable 'dclone';
 
-our $VERSION = '0.25';
+our $VERSION = '0.26';
 our @CARP_NOT = (qw(XML::Twig));
 
 use Filter::signatures;
@@ -182,8 +183,50 @@ This method also takes the same options as the constructor.
 
 =cut
 
-sub parse {
-    my( $self, $source, @options ) = @_;
+sub _empty_cell( $self, $readonly, $is_merged ) {
+    state $merged_cell = Spreadsheet::ParseODS::Cell->new({
+            type         => undef,
+            unformatted  => undef,
+            value        => undef,
+            formula      => undef,
+            hyperlink    => undef,
+            style        => undef,
+            format       => undef,
+            is_merged    => 1,
+            is_hidden    => undef,
+    });
+
+    state $empty_cell = Spreadsheet::ParseODS::Cell->new({
+            type         => undef,
+            unformatted  => undef,
+            value        => undef,
+            formula      => undef,
+            hyperlink    => undef,
+            style        => undef,
+            format       => undef,
+            is_merged    => undef,
+            is_hidden    => undef,
+    });
+
+    if( $readonly ) {
+        return $is_merged ? $merged_cell : $empty_cell
+
+    } else {
+        return Spreadsheet::ParseODS::Cell->new({
+            type         => undef,
+            unformatted  => undef,
+            value        => undef,
+            formula      => undef,
+            hyperlink    => undef,
+            style        => undef,
+            format       => undef,
+            is_merged    => $is_merged,
+            is_hidden    => undef,
+        });
+    }
+}
+
+sub parse( $self, $source, @options ) {
     my %options;
     my $formatter;
     if( @options % 2 == 0 ) {
@@ -389,23 +432,12 @@ sub parse {
                     $text = $unformatted;
                 };
 
-                for my $i (1..$repeat) {
-
                     # Yes, this is somewhat inefficient, but it saves us
                     # from later programming errors if we create/store
                     # references. We can always later turn this inside-out.
+                    my $cell_obj;
                     if( $cell->is_empty ) {
-                        push @$rowref, Spreadsheet::ParseODS::Cell->new({
-                            type         => undef,
-                            unformatted  => undef,
-                            value        => undef,
-                            formula      => undef,
-                            hyperlink    => undef,
-                            style        => undef,
-                            format       => undef,
-                            is_merged    => $is_merged,
-                            is_hidden    => undef,
-                        });
+                        $cell_obj = $self->_empty_cell( $readonly, $is_merged );
 
                     } else {
 
@@ -435,7 +467,7 @@ sub parse {
 
                         };
 
-                        my $cell = Spreadsheet::ParseODS::Cell->new({
+                        $cell_obj = Spreadsheet::ParseODS::Cell->new({
                                   value        => $text,
                                   unformatted  => defined $unformatted ? $unformatted : $text,
                                   formula      => $formula,
@@ -447,9 +479,15 @@ sub parse {
                             maybe 'format'    => $f,
                         });
 
-                        push @$rowref, $cell;
                     };
-                };
+                    if( $readonly ) {
+                        push @$rowref, ($cell_obj) x $repeat;
+                    } else {
+                        push @$rowref, $cell_obj;
+                        for (2..$repeat) {
+                            push @$rowref, (ref $cell_obj)->new( { %$cell_obj } );
+                        };
+                    };
             };
 
             # if number-rows-repeated is set, set $repeat_rows value accordingly for later use
@@ -528,16 +566,18 @@ sub parse {
                             );
         $p->setTwigHandlers( \%style_handlers );
         $p->$method( $xml );
-
         # read /settings.xml in addition, to fill stuff like ActiveSheet
         ($method, $xml) = $self->_open_xml_thing(
                                 $source,
                                 $options,
                                 inputtype => $options{ inputtype },
                                 member_file => 'settings.xml',
+                                optional => 1,
                             );
-        $p->setTwigHandlers( \%setting_handlers );
-        $p->$method( $xml );
+        if( defined $xml) {
+            $p->setTwigHandlers( \%setting_handlers );
+            $p->$method( $xml );
+        };
         # Also maybe read /meta.xml for the remaining information
 
     };
@@ -574,7 +614,7 @@ sub _open_xml_thing( $self, $source, $wb_info, %options ) {
             $xml = $source;
 
         } else {
-            $xml = $self->_open_sxc( $source, \%options )
+            $xml = $self->_open_sxc( $source, \%options );
         };
 
     } else {
@@ -620,17 +660,24 @@ sub _open_sxc {
     };
     open my $fh, '<', $sxc_file
         or croak "Couldn't open '$sxc_file': $!";
-    return $self->_open_sxc_fh( $fh, $options_ref->{member_file} );
+    return $self->_open_sxc_fh( $fh, $options_ref->{member_file},
+        maybe optional => $options_ref->{optional}
+    );
 }
 
-sub _open_sxc_fh($self, $fh, $member) {
+sub _open_sxc_fh($self, $fh, $member, %options) {
     my $zip = Archive::Zip->new();
     my $status = $zip->readFromFileHandle($fh);
     $status == AZ_OK
         or croak "Read error from zip";
-    my $content = $zip->memberNamed($member)
-        or croak "Want to read $member' but it doesn't exist!";
-
+    my $content = $zip->memberNamed($member);
+    if( ! defined $content ) {
+        if( $options{ optional }) {
+            return;
+        } else {
+            croak "Want to read $member' but it doesn't exist!";
+        }
+    }
     $content->rewindData();
     my $stream = $content->fh;
     1 if eof($stream); # reset eof state of $stream?! Is that a bug? Where?
